[inherit('common')]
module xmlrpc_format(input, output);

[external]
function xmlrpc_i4(%immed val : integer) : c_str_t; external;

[external]
function xmlrpc_string(%immed val : c_str_t) : c_str_t; external;

[external]
function xmlrpc_member(%immed nam : c_str_t; %immed val : c_str_t) : c_str_t; external;

[external]
function xmlrpc_struct2(%immed nmemb : integer; %immed memb : [list] c_str_t) : c_str_t; external;

[external]
function xmlrpc_array(%immed nelm : integer; %immed elm : [list] c_str_t) : c_str_t; external;

[external]
procedure xmlrpc_request2(%immed buf : integer; %immed bufsiz : integer; %ref buflen : word; %immed func : c_str_t; %immed narg : integer; %immed params : [list] c_str_t); external;

function pxmlrpc_i4(val : integer) : c_str_t;

begin
   pxmlrpc_i4 := xmlrpc_i4(val);
end;

function pxmlrpc_string(val : pstr) : c_str_t;

var
   temp : c_str_t;

begin
   temp := malloc_c_str(val);
   pxmlrpc_string := xmlrpc_string(temp);
   free_c_str(temp);
end;

function pxmlrpc_member(nam : pstr; val : c_str_t) : c_str_t;

var
   temp : c_str_t;

begin
   temp := malloc_c_str(nam);
   pxmlrpc_member := xmlrpc_member(temp, val);
   free_c_str(temp);
end;

function pxmlrpc_struct2(%immed memb : [list] c_str_t) : c_str_t;

var
   nmemb : integer;

begin
   nmemb := argument_list_length(memb);
   case nmemb of
      0 : pxmlrpc_struct2 := xmlrpc_struct2(nmemb);
      1 : pxmlrpc_struct2 := xmlrpc_struct2(nmemb, argument(memb, 1));
      2 : pxmlrpc_struct2 := xmlrpc_struct2(nmemb, argument(memb, 1), argument(memb, 2));
      3 : pxmlrpc_struct2 := xmlrpc_struct2(nmemb, argument(memb, 1), argument(memb, 2), argument(memb, 3));
      4 : pxmlrpc_struct2 := xmlrpc_struct2(nmemb, argument(memb, 1), argument(memb, 2), argument(memb, 3), argument(memb, 4));
      5 : pxmlrpc_struct2 := xmlrpc_struct2(nmemb, argument(memb, 1), argument(memb, 2), argument(memb, 3), argument(memb, 4), argument(memb, 5));
      otherwise halt;
   end;
end;

function pxmlrpc_array(%immed elm : [list] c_str_t) : c_str_t;

var
   nelm : integer;

begin
   nelm := argument_list_length(elm);
   case nelm of
      0 : pxmlrpc_array := xmlrpc_array(nelm);
      1 : pxmlrpc_array := xmlrpc_array(nelm, argument(elm, 1));
      2 : pxmlrpc_array := xmlrpc_array(nelm, argument(elm, 1), argument(elm, 2));
      3 : pxmlrpc_array := xmlrpc_array(nelm, argument(elm, 1), argument(elm, 2), argument(elm, 3));
      4 : pxmlrpc_array := xmlrpc_array(nelm, argument(elm, 1), argument(elm, 2), argument(elm, 3), argument(elm, 4));
      5 : pxmlrpc_array := xmlrpc_array(nelm, argument(elm, 1), argument(elm, 2), argument(elm, 3), argument(elm, 4), argument(elm, 5));
      otherwise halt;
   end;
end;

procedure pxmlrpc_request(var buf : pstr; func : pstr; params : [list] c_str_t);

var
   func2, param1, param2, param3 : c_str_t;
   nparams : integer;

begin
   func2 := malloc_c_str(func);
   nparams := argument_list_length(params);
   case nparams of
      0: xmlrpc_request2(iaddress(buf.body), 32000, buf.length, func2, 0);
      1: begin
            param1 := argument(params, 1);
            xmlrpc_request2(iaddress(buf.body), 32000, buf.length, func2, 1, param1);
            free_c_str(param1);
         end;
      2: begin
            param1 := argument(params, 1);
            param2 := argument(params, 2);
            xmlrpc_request2(iaddress(buf.body), 32000, buf.length, func2, 2, param1, param2);
            free_c_str(param2);
            free_c_str(param1);
         end;
      3: begin
            param1 := argument(params, 1);
            param2 := argument(params, 2);
            param3 := argument(params, 3);
            xmlrpc_request2(iaddress(buf.body), 32000, buf.length, func2, 3, param1, param2, param3);
            free_c_str(param3);
            free_c_str(param2);
            free_c_str(param1);
         end;
      otherwise halt;
   end;
   free_c_str(func2);
end;

end.
